@@ -1,4 +1,4 @@
-//          Copyright Naoki Shibata 2010 - 2019.
+//   Copyright Naoki Shibata and contributors 2010 - 2020.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -11,14 +11,6 @@
 #include <float.h>
 #include <limits.h>
 
-#if defined(POWER64_UNDEF_USE_EXTERN_INLINES)
-// This is a workaround required to cross compile for PPC64 binaries
-#include <features.h>
-#ifdef __USE_EXTERN_INLINES
-#undef __USE_EXTERN_INLINES
-#endif
-#endif
-
 #include <math.h>
 
 #if defined(__MINGW32__) || defined(__MINGW64__) || defined(_MSC_VER)
@@ -33,6 +25,10 @@
 #include <unistd.h>
 #endif
 
+#if defined(_MSC_VER)
+#include <io.h>
+#endif
+
 #include "misc.h"
 
 #define DENORMAL_DBL_MIN (4.9406564584124654418e-324)
