@@ -1,4 +1,4 @@
-//          Copyright Naoki Shibata 2010 - 2019.
+//   Copyright Naoki Shibata and contributors 2010 - 2020.
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE.txt or copy at
 //          http://www.boost.org/LICENSE_1_0.txt)
@@ -2036,6 +2036,39 @@ EXPORT CONST float xfmodf(float x, float y) {
   return ret;
 }
 
+static INLINE CONST float rintfk2(float d) {
+  float x = d + 0.5f;
+  int32_t isodd = (1 & (int32_t)x) != 0;
+  float fr = x - (int32_t)x;
+  fr = (fr < 0 || (fr == 0 && isodd)) ? fr+1.0f : fr;
+  return (fabsfk(d) >= (float)(1LL << 23)) ? d : copysignfk(x - fr, d);
+}
+
+EXPORT CONST float xremainderf(float x, float y) {
+  float n = fabsfk(x), d = fabsfk(y), s = 1, q;
+  if (d < FLT_MIN*2) { n *= 1ULL << 25; d *= 1ULL << 25; s = 1.0f / (1ULL << 25); }
+  float rd = 1.0f / d;
+  Sleef_float2 r = df(n, 0);
+  int qisodd = 0;
+
+  for(int i=0;i<8;i++) { // ceil(log2(FLT_MAX) / 22)+1
+    q = rintfk2(r.x * rd);
+    if (fabsfk(r.x) < 1.5f * d) q = r.x < 0 ? -1 : 1;
+    if (fabsfk(r.x) < 0.5f * d || (fabsfk(r.x) == 0.5f * d && !qisodd)) q = 0;
+    if (q == 0) break;
+    if (xisinff(q * -d)) q = q + mulsignf(-1, r.x);
+    qisodd ^= (1 & (int)q) != 0 && fabsfk(q) < (float)(1LL << 24);
+    r = dfnormalize_f2_f2(dfadd2_f2_f2_f2(r, dfmul_f2_f_f(q, -d)));
+  }
+  
+  float ret = r.x * s;
+  ret = mulsignf(ret, x);
+  if (xisinff(y)) ret = xisinff(x) ? SLEEF_NANf : x;
+  if (d == 0) ret = SLEEF_NANf;
+
+  return ret;
+}
+
 EXPORT CONST float xsqrtf_u05(float d) {
   float q = 0.5f;
 
